<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kdf" version="1">
<MenuBar>
 <Menu name="file" noMerge="1">
    <text>&amp;File</text>
    <Action name="updatedf"/>
    <Separator />
    <Action name="file_quit"/> 
 </Menu>
</MenuBar>
</gui>
