/*
* kdfitemdelegate.h
*
* Copyright (c) 2009 Dario Andres Rodriguez <andresbajotierra@gmail.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef KDFITEMDELEGATE_H
#define KDFITEMDELEGATE_H

#include <QStyledItemDelegate>

class QPainter;

class KDFItemDelegate: public QStyledItemDelegate
{
    Q_OBJECT

    public:
        explicit KDFItemDelegate(QObject * parent = nullptr): QStyledItemDelegate(parent) {}
        ~KDFItemDelegate() override {}

        void paint(QPainter *painter, const QStyleOptionViewItem &option,
            const QModelIndex &index) const override;

};

#endif
